@@ -23,25 +23,29 @@ module Commands
     , tenthSeconds
     ) where
 
-import Prelude hiding (catch)
+import Prelude
 import Control.Concurrent
-import Control.Exception
+import Control.Exception (handle, SomeException(..))
 import Data.Char
 import System.Process
 import System.Exit
 import System.IO (hClose)
+
+import Signal
 import XUtil
 
 class Show e => Exec e where
-    alias :: e -> String
-    alias e    = takeWhile (not . isSpace) $ show e
-    rate  :: e -> Int
-    rate _     = 10
-    run   :: e -> IO String
-    run _      = return ""
-    start :: e -> (String -> IO ()) -> IO ()
-    start e cb = go
+    alias   :: e -> String
+    alias   e    = takeWhile (not . isSpace) $ show e
+    rate    :: e -> Int
+    rate    _    = 10
+    run     :: e -> IO String
+    run     _    = return ""
+    start   :: e -> (String -> IO ()) -> IO ()
+    start   e cb = go
         where go = run e >>= cb >> tenthSeconds (rate e) >> go
+    trigger :: e -> (Maybe SignalType -> IO ()) -> IO ()
+    trigger _ sh  = sh Nothing
 
 data Command = Com Program Args Alias Rate
                deriving (Show,Read,Eq)
@@ -61,12 +65,12 @@ instance Exec Command where
                 (i,o,e,p) <- runInteractiveCommand (unwords (prog:args))
                 exit <- waitForProcess p
                 let closeHandles = hClose o >> hClose i >> hClose e
+                    getL = handle (\(SomeException _) -> return "")
+                                  (hGetLineSafe o)
                 case exit of
-                  ExitSuccess -> do
-                            str <- catch (hGetLineSafe o)
-                                         (\(SomeException _) -> return "")
-                            closeHandles
-                            cb str
+                  ExitSuccess -> do str <- getL
+                                    closeHandles
+                                    cb str
                   _ -> do closeHandles
                           cb $ "Could not execute command " ++ prog
 
