@@ -28,16 +28,12 @@ cpuFreqConfig = mkMConfig
                                                               -- replacements
 
 -- |
--- Function retrieves monitor string holding the cpu frequency (or
--- frequencies)
+-- Function retrieves monitor string holding the cpu frequency (or frequencies)
 runCpuFreq :: [String] -> Monitor String
-runCpuFreq _ = do
-    let dir = "/sys/devices/system/cpu"
-        file = "cpufreq/scaling_cur_freq"
-        pattern = "cpu"
-        divisor = 1e6 :: Double
-        failureMessage = "CpuFreq: N/A"
-        fmt x | x < 1     = show (round (x * 1000) :: Integer) ++ "MHz"
-              | otherwise = showDigits 1 x ++ "GHz"
-    checkedDataRetrieval failureMessage dir file pattern (/divisor) fmt
+runCpuFreq _ = let path = ["/sys/devices/system/cpu/cpu", "/cpufreq/scaling_cur_freq"]
+                   divisor = 1e6 :: Double
+                   failureMessage = "CpuFreq: N/A"
+                   fmt x | x < 1     = (show (round (x * 1000) :: Integer)) ++ "MHz"
+                         | otherwise = (show x) ++ "GHz"
+               in  checkedDataRetrieval failureMessage path Nothing (/divisor) fmt
 
