@@ -14,21 +14,22 @@
 
 module Plugins.StdinReader where
 
-import Prelude hiding (catch)
+import Prelude
 import System.Posix.Process
 import System.Exit
 import System.IO
-import Control.Exception (SomeException(..),catch)
+import Control.Exception (SomeException(..), handle)
 import Plugins
+import Actions (stripActions)
 
-data StdinReader = StdinReader
-    deriving (Read, Show)
+data StdinReader = StdinReader deriving (Read, Show)
 
 instance Exec StdinReader where
-    start StdinReader cb = do
-        cb =<< catch (hGetLineSafe stdin)
-                     (\(SomeException e) -> do hPrint stderr e; return "")
-        eof <- hIsEOF stdin
-        if eof
-            then exitImmediately ExitSuccess
-            else start StdinReader cb
+  start StdinReader cb = do
+    s <- handle (\(SomeException e) -> do hPrint stderr e; return "")
+                (hGetLineSafe stdin)
+    cb (stripActions s)
+    eof <- hIsEOF stdin
+    if eof
+      then exitImmediately ExitSuccess
+      else start StdinReader cb
